# Function to build a function to put inside compute
build_compute_workflow <- function(recipe) {
  ## TODO: Sort out system and order
  ## Alternatively, this function could just be defined by the user
  # Define function
  my_compute_function = function(recipe, hcst, obs, fcst = NULL) {
    add_calibration <- TRUE
    if (add_calibration) {
      data <- calibrate_datasets(recipe, hcst, obs, fcst)
    }
    return(data)
  }
  info(recipe$Run$logger, "##### COMPUTE FUNCTION DEFINED #####")
  return(my_compute_function)
}

convert_to_s2dv_cube <- function(new_cube, original_cube) {
  ## TODO: Make sure all the metadata is created correctly
  attr(new_cube, "Variables") <- attr(original_cube, "Variables")
  attr(new_cube, "FileSelectors") <- attr(original_cube, "FileSelectors")
  class(new_cube) <- "startR_array"
  new_cube <- as.s2dv_cube(new_cube)
  return(new_cube)
}

run_compute_workflow <- function(recipe, data) {

  # Step 1: Define the function
  ## TODO: Sort out order
  my_compute_function <- build_compute_workflow(recipe)
  # Step 2: Define the call to compute()
  ## TODO: Define the way the dimensions are split

  step <- Step(fun = my_compute_function,
               target_dims = list(c('sday', 'sweek', 'syear', 'ensemble'),
                                  c('sday', 'sweek', 'syear')),
               output_dims = list(c('sday', 'sweek', 'syear', 'ensemble'),
                                  c('sday', 'sweek', 'syear')))
  wf <- AddStep(inputs = list(hcst = data$hcst, obs = data$obs),
                              step,
                              recipe = recipe)
  res <- Compute(wf$output2,
                 chunks = list(latitude = 2,
                               longitude = 2))

  info(recipe$Run$logger,
       "##### COMPUTE SECTION ENDED, CONVERTING TO s2dv_cube #####")
  # Step 3: Convert result to s2dv_cube and return it
  hcst <- convert_to_s2dv_cube(res$output1, data$hcst)
  obs <- convert_to_s2dv_cube(res$output2, data$obs)
  ## TODO: fcst
  info(recipe$Run$logger,
       "##### DATA RETURNED AS A LIST OF s2dv_cube OBJECTS #####")
  return(list(hcst = hcst, obs = obs))
}

